@@ -33,6 +33,12 @@ try:  # Python 3
 except ImportError:  # Python 2
     from urllib2 import HTTPError, Request, URLError, urlopen
 
+try:
+    # On windows, sometimes the python install is missing certificates
+    import certifi  # noqa: F401
+except ImportError:
+    pass
+
 # Clean environment for subprocesses
 clean_env = {
     key: value
@@ -44,7 +50,7 @@ devnull = open(os.devnull, "wb")  # to redirect unwanted subprocess output
 allowed_ssh_connections = {}
 concurrent_git_connection_limit = threading.Semaphore(5)
 
-_prebuilt_cctbx_base = "2022.8"  # September 2022 release
+_prebuilt_cctbx_base = "v2023.9"
 
 
 def make_executable(filepath):
@@ -55,7 +61,7 @@ def make_executable(filepath):
         os.chmod(filepath, mode)
 
 
-def install_micromamba(python, include_cctbx):
+def install_micromamba(python, include_cctbx, cmake):
     """Download and install Micromamba"""
     if sys.platform.startswith("linux"):
         conda_platform = "linux"
@@ -136,10 +142,16 @@ def install_micromamba(python, include_cctbx):
         "--channel",
         "conda-forge",
         "--override-channels",
+        "mamba",
         python_requirement,
     ]
-    if include_cctbx:
-        command_list.append("cctbx-base=" + _prebuilt_cctbx_base)
+    if include_cctbx or cmake:
+        command_list.append("cctbx-nightly::cctbx-base=" + _prebuilt_cctbx_base)
+    if cmake:
+        command_list.extend(["pycbf", "cmake"])
+    if os.name == "nt":
+        # Installing pre-commit via precommittbx does not work on windows
+        command_list.append("pre-commit")
 
     print(
         "{text} dials environment from {filename} with Python {python}".format(
@@ -222,7 +234,7 @@ def install_miniconda(location):
     run_command(command=command, workdir=".")
 
 
-def install_conda(python, include_cctbx):
+def install_conda(python, include_cctbx, cmake):
     # Find relevant conda base installation
     conda_base = os.path.realpath("miniconda")
     if os.name == "nt":
@@ -331,8 +343,10 @@ environments exist and are working.
         "--override-channels",
         python_requirement,
     ]
-    if include_cctbx:
-        command_list.append("cctbx-base=" + _prebuilt_cctbx_base)
+    if include_cctbx or cmake:
+        command_list.append("cctbx-nightly::cctbx-base=" + _prebuilt_cctbx_base)
+    if cmake:
+        command_list.extend(["pycbf", "cmake", "pre-commit"])
     if os.name == "nt":
         command_list = [
             "cmd.exe",
@@ -995,29 +1009,46 @@ def update_sources(options):
         except OSError:
             pass
 
-    repositories = {
-        source.split("/")[1]: {"base-repository": source, "branch-local": "dials-3.12"}
-        for source, branch in (
-            ("dials/annlib_adaptbx", "master"),
-            ("dials/dxtbx", "main"),
-            ("dials/annlib", "master"),
-            ("dials/cbflib", "main"),
-            ("dials/ccp4io", "master"),
-            ("dials/ccp4io_adaptbx", "master"),
-            ("dials/cctbx_project", "master"),
-            ("dials/dials", "main"),
-            ("dials/gui_resources", "master"),
-            ("xia2/xia2", "main"),
-        )
-    }
-    if options.prebuilt_cctbx:
-        repositories["cctbx_project"]["branch-local"] = (
-            "releases/" + _prebuilt_cctbx_base
-        )
+    if not options.cmake:
+        repositories = {
+            source.split("/")[1]: {
+                "base-repository": source,
+                "branch-local": "dials-3.17",
+            }
+            for source, branch in (
+                ("dials/annlib_adaptbx", "master"),
+                ("dials/annlib", "master"),
+                ("dials/cbflib", "main"),
+                ("dials/ccp4io_adaptbx", "master"),
+                ("dials/ccp4io", "master"),
+                ("dials/cctbx_project", "master"),
+                ("dials/dials", "main"),
+                ("dials/dxtbx", "main"),
+                ("dials/gui_resources", "master"),
+                ("xia2/xia2", "main"),
+            )
+        }
+        if options.prebuilt_cctbx:
+            repositories["cctbx_project"]["branch-local"] = (
+                "releases/" + _prebuilt_cctbx_base
+            )
+        else:
+            repositories["cctbx_project"] = {
+                "base-repository": "dials/cctbx_project",
+                "branch-local": "dials-3.17",
+            }
     else:
-        repositories["cctbx_project"] = {
-            "base-repository": "dials/cctbx_project",
-            "branch-local": "dials-3.12",
+        # Only what we need for CMake
+        repositories = {
+            source.split("/")[1]: {
+                "base-repository": source,
+                "branch-local": "dials-3.17",
+            }
+            for source, branch in (
+                ("dials/dxtbx", "main"),
+                ("dials/dials", "main"),
+                ("xia2/xia2", "main"),
+            )
         }
 
     for source, setting in options.branch:
@@ -1116,23 +1147,150 @@ def refresh_build(prebuilt_cctbx):
     )
 
 
-def install_precommit():
+def install_precommit(cmake=False):
     print("Installing precommits")
-    run_indirect_command(
-        os.path.join("bin", "libtbx.precommit"),
-        args=["install"],
-    )
+    if cmake:
+        # Just find all repository directories under modules, since we
+        # don't have a central registry
+        subdirs = []
+        for sub in os.listdir("modules"):
+            if os.path.isdir(os.path.join("modules", sub, ".git")):
+                subdirs.append(os.path.abspath(os.path.join("modules", sub)))
+
+        conda_base_root = os.path.join(os.path.abspath("."), "conda_base")
+        if os.name == "nt":
+            precommit_command = os.path.join(
+                conda_base_root, "Scripts", "libtbx.precommit.exe"
+            )
+        else:
+            precommit_command = os.path.join(conda_base_root, "bin", "libtbx.precommit")
+        run_indirect_command(
+            precommit_command,
+            args=["install"] + subdirs,
+        )
+    else:
+        run_indirect_command(
+            os.path.join("bin", "libtbx.precommit"),
+            args=["install"],
+        )
 
 
-def configure_build(config_flags, prebuilt_cctbx):
+def _get_base_python():
     if os.name == "nt":
         conda_python = os.path.join(os.getcwd(), "conda_base", "python.exe")
     elif sys.platform.startswith("darwin"):
         conda_python = os.path.join(
-            "..", "conda_base", "python.app", "Contents", "MacOS", "python"
+            "conda_base", "python.app", "Contents", "MacOS", "python"
         )
     else:
-        conda_python = os.path.join("..", "conda_base", "bin", "python")
+        conda_python = os.path.join("conda_base", "bin", "python")
+    return os.path.abspath(conda_python)
+
+
+def _get_cmake_exe():
+    if os.name == "nt":
+        return os.path.abspath(
+            os.path.join("conda_base", "Library", "bin", "cmake.exe")
+        )
+    else:
+        return os.path.abspath(os.path.join("conda_base", "bin", "cmake"))
+
+
+def refresh_build_cmake():
+    conda_python = _get_base_python()
+    run_indirect_command(conda_python, ["-mpip", "install", "-e", "../modules/dxtbx"])
+    run_indirect_command(conda_python, ["-mpip", "install", "-e", "../modules/dials"])
+    run_indirect_command(conda_python, ["-mpip", "install", "-e", "../modules/xia2"])
+
+
+def configure_build_cmake():
+    cmake_exe = _get_cmake_exe()
+    python_exe = _get_base_python()
+
+    # Get the location of site-packages
+    site_path = subprocess.check_output(
+        [
+            python_exe,
+            "-c",
+            "import os, site; print(os.path.abspath(site.getsitepackages()[0]))",
+        ]
+    ).strip()
+    if not isinstance(site_path, str):
+        site_path = site_path.decode()
+
+    # Write a .pth file here pointing to the build/lib folder. This
+    # is a somewhat "clever" convenience such that the PYTHONPATH doesn't
+    # need to be explicitly set before running code.
+    lib_pth = os.path.join(site_path, "__bootstrap__.dials.pth")
+    print("Writing libdir .pth file to: " + lib_pth)
+    # Best-guess work out where the libraries are put by the build
+    if os.name == "nt":
+        build_lib_dir = os.path.join(os.getcwd(), "build", "lib", "RelWithDebInfo")
+    else:
+        build_lib_dir = os.path.join(os.getcwd(), "build", "lib")
+
+    with open(lib_pth, "w") as f:
+        f.write(build_lib_dir)
+
+    # write a new-style environment setup script
+    if os.name == "nt":
+        activate = os.path.join(os.getcwd(), "conda_base", "condabin", "activate.bat")
+        with open("dials.bat", "w") as f:
+            f.write(
+                """\
+rem enable conda environment
+call {}
+conda activate {}
+""".format(
+                    activate,
+                    os.path.join(os.getcwd(), "conda_base"),
+                )
+            )
+    else:
+        with open("dials", "w") as f:
+            f.write(
+                """\
+# enable conda environment
+source {dist_root}/conda_base/etc/profile.d/conda.sh
+conda activate {dist_root}/conda_base
+""".format(
+                    dist_root=os.getcwd(),
+                    build_lib=os.path.join(os.getcwd(), "build", "lib"),
+                )
+            )
+
+    # Write a compound CMakeLists.txt, if one doesn't exist
+    if not os.path.isfile("modules/CMakeLists.txt"):
+        with open("modules/CMakeLists.txt", "w") as f:
+            f.write(
+                """\
+cmake_minimum_required(VERSION 3.20 FATAL_ERROR)
+project(dials)
+
+add_subdirectory(dxtbx)
+add_subdirectory(dials)
+"""
+            )
+
+    # run_indirect runs inside the build folder with an activated environment
+    conda_base_root = os.path.join(os.path.abspath("."), "conda_base")
+    assert os.path.isdir(conda_base_root)
+    extra_args = []
+    if os.name == "nt":
+        extra_args.append("-DPython_ROOT_DIR=" + conda_base_root)
+    run_indirect_command(
+        cmake_exe,
+        [
+            "../modules",
+            "-DCMAKE_INSTALL_PREFIX=" + conda_base_root,
+            "-DHDF5_ROOT=" + conda_base_root,
+        ]
+        + extra_args,
+    )
+
+
+def configure_build(config_flags, prebuilt_cctbx):
+    conda_python = _get_base_python()
 
     # write a new-style environment setup script
     with open(("dials.bat" if os.name == "nt" else "dials"), "w"):
@@ -1142,10 +1300,11 @@ def configure_build(config_flags, prebuilt_cctbx):
         flag.startswith("--compiler=") for flag in config_flags
     ):
         config_flags.append("--compiler=conda")
-    if "--enable_cxx11" not in config_flags:
-        config_flags.append("--enable_cxx11")
     if "--use_conda" not in config_flags:
         config_flags.append("--use_conda")
+    # Default to C++14 if otherwise unspecified
+    if not any(x.startswith("--cxxstd") for x in config_flags):
+        config_flags.append("--cxxstd=c++14")
 
     print("Setting up build directory")
     if prebuilt_cctbx:
@@ -1196,6 +1355,14 @@ def make_build(prebuilt_cctbx):
         run_indirect_command(command, args=["-j", str(nproc)])
 
 
+def make_build_cmake():
+    cmake_exe = _get_cmake_exe()
+    if os.name == "nt":
+        run_indirect_command(cmake_exe, ["--build", ".", "--config", "RelWithDebInfo"])
+    else:
+        run_indirect_command(cmake_exe, ["--build", "."])
+
+
 def repository_at_tag(string):
     try:
         repository, tag = string.split("@", 1)
@@ -1268,8 +1435,8 @@ be passed separately with quotes to avoid confusion (e.g
     parser.add_argument(
         "--python",
         help="Install this minor version of Python (default: %(default)s)",
-        default="3.9",
-        choices=("3.8", "3.9", "3.10"),
+        default="3.10",
+        choices=("3.9", "3.10", "3.11"),
     )
     parser.add_argument(
         "--branch",
@@ -1301,6 +1468,11 @@ be passed separately with quotes to avoid confusion (e.g
         default=False,
         action="store_true",
     )
+    parser.add_argument(
+        "--cmake",
+        help="Use the CMake build system. Implies --prebuilt-cctbx.",
+        action="store_true",
+    )
 
     options = parser.parse_args()
 
@@ -1313,20 +1485,33 @@ be passed separately with quotes to avoid confusion (e.g
     # Build base packages
     if "base" in options.actions:
         if options.conda:
-            install_conda(options.python, include_cctbx=options.prebuilt_cctbx)
+            install_conda(
+                options.python,
+                include_cctbx=options.prebuilt_cctbx,
+                cmake=options.cmake,
+            )
             if options.clean:
                 shutil.rmtree(os.path.realpath("miniconda"))
         else:
-            install_micromamba(options.python, include_cctbx=options.prebuilt_cctbx)
+            install_micromamba(
+                options.python,
+                include_cctbx=options.prebuilt_cctbx,
+                cmake=options.cmake,
+            )
             if options.clean:
                 shutil.rmtree(os.path.realpath("micromamba"))
 
     # Configure, make, get revision numbers
     if "build" in options.actions:
-        configure_build(options.config_flags, prebuilt_cctbx=options.prebuilt_cctbx)
-        make_build(prebuilt_cctbx=options.prebuilt_cctbx)
-        refresh_build(prebuilt_cctbx=options.prebuilt_cctbx)
-        install_precommit()
+        if options.cmake:
+            refresh_build_cmake()
+            configure_build_cmake()
+            make_build_cmake()
+        else:
+            configure_build(options.config_flags, prebuilt_cctbx=options.prebuilt_cctbx)
+            make_build(prebuilt_cctbx=options.prebuilt_cctbx)
+            refresh_build(prebuilt_cctbx=options.prebuilt_cctbx)
+        install_precommit(options.cmake)
 
     # Tests, tests
     if "test" in options.actions:
