@@ -171,7 +171,7 @@ indexing {
       .help = "refine_shells: refine in increasing resolution cutoffs after indexing."
               "repredict_only: do not refine after indexing, just update spot"
               "predictions."
-              "None: turns off all forms of refinement (currently only applies to stills_indexer)"
+              "None: do not refine and do not update spot predictions."
     n_macro_cycles = 5
       .type = int(value_min=1)
       .help = "Maximum number of macro cycles of refinement, reindexing all"
@@ -518,7 +518,7 @@ class Indexer:
             if had_refinement_error or have_similar_crystal_models:
                 break
             max_lattices = self.params.multiple_lattice_search.max_lattices
-            if max_lattices is not None and len(experiments) >= max_lattices:
+            if max_lattices is not None and len(experiments.crystals()) >= max_lattices:
                 break
             if len(experiments) > 0:
                 cutoff_fraction = (
@@ -642,6 +642,11 @@ class Indexer:
                         spherical_relp=self.all_params.refinement.parameterisation.spherical_relp_model,
                     )
                     ref_predictor(refined_reflections)
+                elif self.params.refinement_protocol.mode is None:
+                    refined_experiments, refined_reflections = (
+                        experiments,
+                        reflections_for_refinement,
+                    )
                 else:
                     try:
                         refined_experiments, refined_reflections = self.refine(
@@ -653,18 +658,35 @@ class Indexer:
                         had_refinement_error = True
                         logger.info("Refinement failed:")
                         logger.info(e)
-                        del experiments[-1]
-
-                        # remove experiment id from the reflections associated
-                        # with this deleted experiment - indexed flag removed
-                        # below
-                        last = len(experiments)
-                        sel = refined_reflections["id"] == last
-                        logger.info(
-                            "Removing %d reflections with id %d", sel.count(True), last
+                        # need to remove crystals - may be shared!
+                        models_to_remove = experiments.where(
+                            crystal=experiments[-1].crystal
                         )
-                        refined_reflections["id"].set_selected(sel, -1)
-
+                        for model_id in sorted(models_to_remove, reverse=True):
+                            del experiments[model_id]
+                            # remove experiment id from the reflections associated
+                            # with this deleted experiment - indexed flag removed
+                            # below
+                            # note here we are acting on the table from the last macrocycle
+                            # This is guaranteed to exist due to the check if len(experiments) == 1: above
+                            sel = self.refined_reflections["id"] == model_id
+                            if sel.count(
+                                True
+                            ):  # not the case if failure on first cycle of refinement of new xtal
+                                logger.info(
+                                    "Removing %d reflections with id %d",
+                                    sel.count(True),
+                                    model_id,
+                                )
+                                self.refined_reflections["id"].set_selected(sel, -1)
+                                # N.B. Need to unset the flags here as the break below means we
+                                # don't enter the code after
+                                del self.refined_reflections.experiment_identifiers()[
+                                    model_id
+                                ]
+                                self.refined_reflections.unset_flags(
+                                    sel, self.refined_reflections.flags.indexed
+                                )
                         break
 
                 self._unit_cell_volume_sanity_check(experiments, refined_experiments)
@@ -789,14 +811,12 @@ class Indexer:
             )
             min_angle = self.params.multiple_lattice_search.minimum_angular_separation
             if abs(angle) < min_angle:  # degrees
+                # account for potential shared crystal model
+                models_to_reject = experiments.where(crystal=cryst_b)
+                reject = len(experiments.crystals())
+                logger.info(f"Crystal models too similar, rejecting crystal {reject}")
                 logger.info(
-                    "Crystal models too similar, rejecting crystal %i:",
-                    len(experiments),
-                )
-                logger.info(
-                    "Rotation matrix to transform crystal %i to crystal %i",
-                    i_a + 1,
-                    len(experiments),
+                    f"Rotation matrix to transform crystal {i_a + 1} to crystal {reject}"
                 )
                 logger.info(R_ab)
                 logger.info(
@@ -804,7 +824,8 @@ class Indexer:
                     + " about axis (%.3f, %.3f, %.3f)" % axis
                 )
                 have_similar_crystal_models = True
-                del experiments[-1]
+                for id_ in sorted(models_to_reject, reverse=True):
+                    del experiments[id_]
                 break
         return have_similar_crystal_models
 
