@@ -109,7 +109,7 @@ class StillsIndexer(Indexer):
         while True:
             self.d_min = self.params.refinement_protocol.d_min_start
             max_lattices = self.params.multiple_lattice_search.max_lattices
-            if max_lattices is not None and len(experiments) >= max_lattices:
+            if max_lattices is not None and len(experiments.crystals()) >= max_lattices:
                 break
             if len(experiments) > 0:
                 cutoff_fraction = (
@@ -363,7 +363,12 @@ class StillsIndexer(Indexer):
                         raise DialsIndexRefineError(e)
                     logger.info("Refinement failed:")
                     logger.info(s)
-                    del experiments[-1]
+                    # need to remove crystals - may be shared?!
+                    models_to_remove = experiments.where(
+                        crystal=experiments[-1].crystal
+                    )
+                    for model_id in sorted(models_to_remove, reverse=True):
+                        del experiments[model_id]
                     break
 
             self._unit_cell_volume_sanity_check(experiments, refined_experiments)
@@ -698,7 +703,9 @@ class StillsIndexer(Indexer):
         from rstbx.indexing_api import outlier_detection
 
         od = outlier_detection.find_outliers_from_matches(
-            matches, verbose=True, horizon_phil=hardcoded_phil
+            matches,
+            verbose=self.all_params.refinement.reflections.outlier.sauter_poon.verbose,
+            horizon_phil=hardcoded_phil,
         )
 
         if hardcoded_phil.indexing.outlier_detection.pdf is not None:
