@@ -560,7 +560,7 @@ def _finalize_stills(reflections, experiments, params):
 
     # verify sigmas are sensible
     if "intensity.prf.value" in integrated:
-        if (integrated["intensity.prf.variance"] <= 0).count(True) > 0:
+        if (integrated["intensity.prf.variance"] < 0).count(True) > 0:
             raise Sorry(
                 "Found negative variances (prf). Are bad pixels properly masked out?"
             )
@@ -616,6 +616,8 @@ class ProfileModellerExecutor(Executor):
     The class to do profile modelling calculations
     """
 
+    __getstate_manages_dict__ = 1
+
     def __init__(self, experiments, profile_fitter):
         """
         Initialise the executor
@@ -719,6 +721,8 @@ class ProfileValidatorExecutor(Executor):
     The class to do profile validation calculations
     """
 
+    __getstate_manages_dict__ = 1
+
     def __init__(self, experiments, profile_fitter):
         """
         Initialise the executor
@@ -822,6 +826,8 @@ class IntegratorExecutor(Executor):
     The class to process the integration data
     """
 
+    __getstate_manages_dict__ = 1
+
     def __init__(
         self, experiments, profile_fitter=None, valid_foreground_threshold=0.75
     ):
