@@ -12,6 +12,7 @@
 #ifndef DIALS_ALGORITHMS_INTEGRATION_PARALLEL_INTEGRATOR_H
 #define DIALS_ALGORITHMS_INTEGRATION_PARALLEL_INTEGRATOR_H
 
+#include <atomic>
 #include <boost/ptr_container/ptr_vector.hpp>
 #include <dxtbx/model/beam.h>
 #include <dxtbx/model/detector.h>
@@ -25,7 +26,8 @@
 #include <dials/algorithms/shoebox/find_overlapping.h>
 #include <dials/algorithms/integration/sum/summation.h>
 #include <dials/algorithms/centroid/centroid.h>
-#include <dials/array_family/boost_python/flex_table_suite.h>
+#include <dxtbx/array_family/flex_table_suite.h>
+#include <dials/array_family/boost_python/reflection_table_suite.h>
 #include <map>
 
 #include <dials/algorithms/integration/interfaces.h>
@@ -946,7 +948,7 @@ namespace dials { namespace algorithms {
 
         // Initialise the vector of atomic ints
         for (std::size_t i = 0; i < num_images; ++i) {
-          counter_.push_back(new boost::atomic<int>(0));
+          counter_.push_back(new std::atomic<int>(0));
         }
 
         // Increment the counter for each image
@@ -1011,7 +1013,7 @@ namespace dials { namespace algorithms {
 
     protected:
       int first_image_;
-      boost::ptr_vector<boost::atomic<int> > counter_;
+      boost::ptr_vector<std::atomic<int> > counter_;
     };
 
     /**
@@ -1604,6 +1606,9 @@ namespace dials { namespace algorithms {
      * @returns The split reflection table
      */
     af::reflection_table split_reflections(af::reflection_table data) const {
+      using dxtbx::af::flex_table_suite::reorder;
+      using dxtbx::af::flex_table_suite::setitem_column;
+
       // Check input
       DIALS_ASSERT(data.is_consistent());
       DIALS_ASSERT(data.size() > 0);
@@ -1642,13 +1647,11 @@ namespace dials { namespace algorithms {
       data.resize(bbox_new.size());
 
       // Reorder the reflections
-      af::boost_python::flex_table_suite::reorder(data, indices.const_ref());
+      reorder(data, indices.const_ref());
 
       // Set the new bounding boxes
-      af::boost_python::flex_table_suite::setitem_column(
-        data, "bbox", bbox_new.const_ref());
-      af::boost_python::flex_table_suite::setitem_column(
-        data, "partial_id", indices.const_ref());
+      setitem_column(data, "bbox", bbox_new.const_ref());
+      setitem_column(data, "partial_id", indices.const_ref());
 
       // Return the data
       return data;
@@ -1660,7 +1663,7 @@ namespace dials { namespace algorithms {
      * @returns The split reflection table
      */
     af::reflection_table select_in_range_reflections(af::reflection_table data) const {
-      using namespace af::boost_python::flex_table_suite;
+      using dials::af::boost_python::reflection_table_suite::select_rows_index;
 
       // Check if any need to be removed
       af::const_ref<int6> bbox = data["bbox"];
@@ -1834,7 +1837,8 @@ namespace dials { namespace algorithms {
      * @returns The reflections for a particular block.
      */
     af::reflection_table split(std::size_t index) {
-      using namespace af::boost_python::flex_table_suite;
+      using dials::af::boost_python::reflection_table_suite::extend;
+      using dials::af::boost_python::reflection_table_suite::select_rows_index;
       DIALS_ASSERT(index < finished_.size());
 
       // Get the job range
@@ -1904,7 +1908,7 @@ namespace dials { namespace algorithms {
      * Accumulate the results.
      */
     void accumulate(std::size_t index, af::reflection_table result) {
-      using namespace af::boost_python::flex_table_suite;
+      using dxtbx::af::flex_table_suite::set_selected_rows_index;
       DIALS_ASSERT(index < finished_.size());
       DIALS_ASSERT(finished_[index] == false);
 
