@@ -7,8 +7,4 @@ from dials_algorithms_profile_model_ellipsoid_ext import *  # noqa: F401, F403;
 
 
 def mosaicity_from_eigen_decomposition(eigen_values):
-    return (
-        sqrt(eigen_values[0]) * 180.0 / pi,
-        sqrt(eigen_values[1]) * 180.0 / pi,
-        sqrt(eigen_values[2]) * 180.0 / pi,
-    )
+    return tuple(sqrt(e) * 180.0 / pi if e > 0 else 0.0 for e in eigen_values)
