@@ -261,10 +261,6 @@ class RefinerFactory:
 
     @classmethod
     def from_parameters_data_experiments(cls, params, reflections, experiments):
-        # TODO Checks on the input
-        # E.g. does every experiment contain at least one overlapping model with at
-        # least one other experiment? Are all the experiments either rotation series
-        # or stills (the combination of both not yet supported)?
 
         # copy the experiments
         experiments = _copy_experiments_for_refining(experiments)
@@ -272,11 +268,34 @@ class RefinerFactory:
         # copy and filter the reflections
         reflections = cls._filter_reflections(reflections)
 
-        return cls._build_components(params, reflections, experiments)
+        (
+            experiments,
+            refman,
+            ref_predictor,
+            do_stills,
+        ) = cls._build_reflection_manager_and_predictor(
+            params, reflections, experiments
+        )
+
+        return cls._build_refiner(params, experiments, refman, ref_predictor, do_stills)
 
     @classmethod
-    def _build_components(cls, params, reflections, experiments):
-        """low level build"""
+    def reflections_after_outlier_rejection(cls, params, reflections, experiments):
+
+        # copy the experiments
+        experiments = _copy_experiments_for_refining(experiments)
+
+        # copy and filter the reflections
+        reflections = cls._filter_reflections(reflections)
+
+        experiments, refman, _, _ = cls._build_reflection_manager_and_predictor(
+            params, reflections, experiments
+        )
+
+        return refman.get_obs()
+
+    @classmethod
+    def _build_reflection_manager_and_predictor(cls, params, reflections, experiments):
 
         # Currently a refinement job can only have one parameterisation of the
         # prediction equation. This can either be of the XYDelPsi (stills) type, the
@@ -348,7 +367,6 @@ class RefinerFactory:
 
         # configure use of sparse data types
         params = cls.config_sparse(params, experiments)
-        do_sparse = params.refinement.parameterisation.sparse
 
         # create managed reflection predictor
         ref_predictor = ExperimentsPredictorFactory.from_experiments(
@@ -379,6 +397,14 @@ class RefinerFactory:
         # the reflection manager
         refman.finalise(analysis)
 
+        return (experiments, refman, ref_predictor, do_stills)
+
+    @classmethod
+    def _build_refiner(cls, params, experiments, refman, ref_predictor, do_stills):
+        """low level build"""
+
+        do_sparse = params.refinement.parameterisation.sparse
+
         # Create model parameterisations
         logger.debug("Building prediction equation parameterisation")
         pred_param = build_prediction_parameterisation(
@@ -717,6 +743,8 @@ class Refiner:
         # Keep track of whether this is stills or scans type refinement
         self.experiment_type = refman.experiment_type
 
+        self._exp_rmsd_table_data = None
+
         return
 
     def get_experiments(self):
@@ -865,10 +893,9 @@ class Refiner:
 
         return
 
-    def print_exp_rmsd_table(self):
-        """print useful output about refinement steps in the form of a simple table"""
-
-        logger.info("\nRMSDs by experiment:")
+    def calc_exp_rmsd_table(self):
+        if self._exp_rmsd_table_data:
+            return self._exp_rmsd_table_data
 
         header = ["Exp\nid", "Nref"]
         for (name, units) in zip(self._target.rmsd_names, self._target.rmsd_units):
@@ -923,6 +950,16 @@ class Refiner:
                     rmsds.append(rmsd * RAD2DEG)
             rows.append([str(iexp), str(num)] + [f"{r:.5g}" for r in rmsds])
 
+        self._exp_rmsd_table_data = (header, rows)
+        return self._exp_rmsd_table_data
+
+    def print_exp_rmsd_table(self):
+        """print useful output about refinement steps in the form of a simple table"""
+
+        logger.info("\nRMSDs by experiment:")
+
+        header, rows = self.calc_exp_rmsd_table()
+
         if len(rows) > 0:
             logger.info(dials.util.tabulate(rows, header))
 
