@@ -61,6 +61,12 @@ def prepare_input(params, experiments, reflections):
 
     #### First exclude any datasets, before the dataset is split into
     #### individual reflection tables and expids set.
+    if (params.dataset_selection.include_datasets is not None) and (
+        params.dataset_selection.use_datasets is None
+    ):
+        params.dataset_selection.use_datasets = (
+            params.dataset_selection.include_datasets
+        )
     if (
         params.dataset_selection.exclude_datasets
         or params.dataset_selection.use_datasets
@@ -175,6 +181,8 @@ def prepare_input(params, experiments, reflections):
             params.scaling_options.reference,
             params.anomalous,
             d_min=d_min_for_structure_model,
+            k_sol=params.scaling_options.reference_model.k_sol,
+            b_sol=params.scaling_options.reference_model.b_sol,
         )
         experiments.append(expt)
         reflections.append(reflection_table)
@@ -284,7 +292,7 @@ class ScalingAlgorithm:
             or self.params.scaling_options.only_target
         ):
             # now remove things that were used as the target:
-            n_target = len(self.experiments) - len(self.scaler.active_scalers)
+            n_target = len(self.experiments) - self.scaler.n_initial_active_scalers
             self.experiments = self.experiments[:-n_target]
             self.reflections = self.reflections[:-n_target]
         # remove any bad datasets:
@@ -318,6 +326,7 @@ class ScalingAlgorithm:
                 self.scaled_miller_array,
                 self.params.output.merging.nbins,
                 self.params.output.use_internal_variance,
+                additional_stats=self.params.output.additional_stats,
             )
         except DialsMergingStatisticsError as e:
             logger.warning(e, exc_info=True)
