@@ -5,7 +5,9 @@ from __future__ import annotations
 import logging
 import pickle
 import sys
-from collections import namedtuple
+from collections import defaultdict, namedtuple
+
+from orderedset import OrderedSet
 
 import dxtbx.model.compare as compare
 import libtbx.phil
@@ -15,6 +17,7 @@ from dxtbx.model.experiment_list import (
     ExperimentList,
     ExperimentListFactory,
 )
+from dxtbx.sequence_filenames import template_regex_from_list
 
 from dials.util import Sorry, log, show_mail_handle_errors
 from dials.util.multi_dataset_handling import generate_experiment_identifiers
@@ -293,6 +296,7 @@ class ReferenceGeometryUpdater:
         # Load reference geometry
         reference_detector = None
         reference_beam = None
+        reference_goniometer = None
         if params.input.reference_geometry is not None:
             from dxtbx.serialize import load
 
@@ -302,20 +306,23 @@ class ReferenceGeometryUpdater:
             )
             assert experiments, "Could not import reference geometry"
             assert len(experiments.detectors()) >= 1
-            assert len(experiments.beams()) >= 1
             if len(experiments.detectors()) > 1:
                 raise Sorry(
                     "The reference geometry file contains %d detector definitions, but only a single definition is allowed."
                     % len(experiments.detectors())
                 )
-            if len(experiments.beams()) > 1:
-                raise Sorry(
-                    "The reference geometry file contains %d beam definitions, but only a single definition is allowed."
-                    % len(experiments.beams())
-                )
             reference_detector = experiments.detectors()[0]
-            reference_beam = experiments.beams()[0]
-            reference_goniometer = experiments.goniometers()[0]
+            if self.params.input.use_beam_reference:
+                assert len(experiments.beams()) >= 1
+                if len(experiments.beams()) > 1:
+                    raise Sorry(
+                        "The reference geometry file contains %d beam definitions, but only a single definition is allowed."
+                        % len(experiments.beams())
+                    )
+                reference_beam = experiments.beams()[0]
+            if self.params.input.use_gonio_reference:
+                assert len(experiments.goniometers()) >= 1
+                reference_goniometer = experiments.goniometers()[0]
         Reference = namedtuple("Reference", ["detector", "beam", "goniometer"])
         return Reference(
             detector=reference_detector,
@@ -334,6 +341,7 @@ class ManualGeometryUpdater:
         Save the params
         """
         self.params = params
+        self.touched = set()
 
     def __call__(self, imageset):
         """
@@ -354,20 +362,29 @@ class ManualGeometryUpdater:
             for j in range(len(imageset)):
                 imageset.set_scan(None, j)
                 imageset.set_goniometer(None, j)
-        if not isinstance(imageset, ImageSequence):
-            if self.params.geometry.convert_stills_to_sequences:
-                imageset = self.convert_stills_to_sequence(imageset)
+
+        beam = imageset.get_beam()
+        detector = imageset.get_detector()
+        goniometer = imageset.get_goniometer()
+        scan = imageset.get_scan()
+
+        # Create a new model with updated geometry for each model that is not
+        # already in the touched set
         if isinstance(imageset, ImageSequence):
-            beam = BeamFactory.from_phil(self.params.geometry, imageset.get_beam())
-            detector = DetectorFactory.from_phil(
-                self.params.geometry, imageset.get_detector(), beam
-            )
-            goniometer = GoniometerFactory.from_phil(
-                self.params.geometry, imageset.get_goniometer()
-            )
-            scan = ScanFactory.from_phil(
-                self.params.geometry, deepcopy(imageset.get_scan())
-            )
+            if beam and beam not in self.touched:
+                beam = BeamFactory.from_phil(self.params.geometry, imageset.get_beam())
+            if detector and detector not in self.touched:
+                detector = DetectorFactory.from_phil(
+                    self.params.geometry, imageset.get_detector(), beam
+                )
+            if goniometer and goniometer not in self.touched:
+                goniometer = GoniometerFactory.from_phil(
+                    self.params.geometry, imageset.get_goniometer()
+                )
+            if scan and scan not in self.touched:
+                scan = ScanFactory.from_phil(
+                    self.params.geometry, deepcopy(imageset.get_scan())
+                )
             i0, i1 = scan.get_array_range()
             j0, j1 = imageset.get_scan().get_array_range()
             if i0 < j0 or i1 > j1:
@@ -385,18 +402,37 @@ class ManualGeometryUpdater:
                 imageset.set_scan(scan)
         else:
             for i in range(len(imageset)):
-                beam = BeamFactory.from_phil(self.params.geometry, imageset.get_beam(i))
-                detector = DetectorFactory.from_phil(
-                    self.params.geometry, imageset.get_detector(i), beam
-                )
-                goniometer = GoniometerFactory.from_phil(
-                    self.params.geometry, imageset.get_goniometer(i)
-                )
-                scan = ScanFactory.from_phil(self.params.geometry, imageset.get_scan(i))
+                if beam and beam not in self.touched:
+                    beam = BeamFactory.from_phil(
+                        self.params.geometry, imageset.get_beam(i)
+                    )
+                if detector and detector not in self.touched:
+                    detector = DetectorFactory.from_phil(
+                        self.params.geometry, imageset.get_detector(i), beam
+                    )
+                if goniometer and goniometer not in self.touched:
+                    goniometer = GoniometerFactory.from_phil(
+                        self.params.geometry, imageset.get_goniometer(i)
+                    )
+                if scan and scan not in self.touched:
+                    scan = ScanFactory.from_phil(
+                        self.params.geometry, imageset.get_scan(i)
+                    )
                 imageset.set_beam(beam, i)
                 imageset.set_detector(detector, i)
                 imageset.set_goniometer(goniometer, i)
                 imageset.set_scan(scan, i)
+
+        # Add the models from this imageset to the touched set, so they will not
+        # have their geometry updated again
+        if beam:
+            self.touched.add(beam)
+        if detector:
+            self.touched.add(detector)
+        if goniometer:
+            self.touched.add(goniometer)
+        if scan:
+            self.touched.add(scan)
         return imageset
 
     def extrapolate_imageset(
@@ -417,59 +453,6 @@ class ManualGeometryUpdater:
         )
         return sequence
 
-    def convert_stills_to_sequence(self, imageset):
-        from dxtbx.model import Scan
-
-        assert self.params.geometry.scan.oscillation is not None
-        beam = imageset.get_beam(index=0)
-        detector = imageset.get_detector(index=0)
-        goniometer = imageset.get_goniometer(index=0)
-        for i in range(1, len(imageset)):
-            b_i = imageset.get_beam(i)
-            d_i = imageset.get_detector(i)
-            g_i = imageset.get_goniometer(i)
-            assert (beam is None and b_i is None) or beam.is_similar_to(
-                imageset.get_beam(index=i),
-                wavelength_tolerance=self.params.input.tolerance.beam.wavelength,
-                direction_tolerance=self.params.input.tolerance.beam.direction,
-                polarization_normal_tolerance=self.params.input.tolerance.beam.polarization_normal,
-                polarization_fraction_tolerance=self.params.input.tolerance.beam.polarization_fraction,
-            )
-            assert (detector is None and d_i is None) or detector.is_similar_to(
-                imageset.get_detector(index=i),
-                fast_axis_tolerance=self.params.input.tolerance.detector.fast_axis,
-                slow_axis_tolerance=self.params.input.tolerance.detector.slow_axis,
-                origin_tolerance=self.params.input.tolerance.detector.origin,
-            )
-            assert (goniometer is None and g_i is None) or goniometer.is_similar_to(
-                imageset.get_goniometer(index=i),
-                rotation_axis_tolerance=self.params.input.tolerance.goniometer.rotation_axis,
-                fixed_rotation_tolerance=self.params.input.tolerance.goniometer.fixed_rotation,
-                setting_rotation_tolerance=self.params.input.tolerance.goniometer.setting_rotation,
-            )
-        oscillation = self.params.geometry.scan.oscillation
-        from dxtbx.imageset import ImageSetFactory
-        from dxtbx.sequence_filenames import template_regex_from_list
-
-        template, indices = template_regex_from_list(imageset.paths())
-        image_range = (min(indices), max(indices))
-        assert (image_range[1] + 1 - image_range[0]) == len(indices)
-        scan = Scan(image_range=image_range, oscillation=oscillation)
-        if template is None:
-            paths = [imageset.get_path(i) for i in range(len(imageset))]
-            assert len(set(paths)) == 1
-            template = paths[0]
-        new_sequence = ImageSetFactory.make_sequence(
-            template=template,
-            indices=indices,
-            format_class=imageset.reader().get_format_class(),
-            beam=beam,
-            detector=detector,
-            goniometer=goniometer,
-            scan=scan,
-        )
-        return new_sequence
-
 
 class MetaDataUpdater:
     """
@@ -513,7 +496,6 @@ class MetaDataUpdater:
         """
         # Import the lookup data
         lookup = self.import_lookup_data(self.params)
-
         # Convert all to ImageGrid
         if self.params.input.as_grid_scan:
             imageset_list = self.convert_to_grid_scan(imageset_list, self.params)
@@ -572,12 +554,12 @@ class MetaDataUpdater:
                     # that these are in people numbers (1...) and are inclusive
                     if self.params.geometry.scan.image_range:
                         user_start, user_end = self.params.geometry.scan.image_range
-                        offset = imageset.get_scan().get_array_range()[0]
                         start, end = user_start - 1, user_end
                     else:
                         start, end = imageset.get_scan().get_array_range()
-                        offset = 0
 
+                    # offset to get 0-based indexing into the imageset
+                    offset = imageset.get_scan().get_array_range()[0]
                     for j in range(start, end):
                         subset = imageset[j - offset : j - offset + 1]
                         experiments.append(
@@ -615,6 +597,68 @@ class MetaDataUpdater:
                         )
                     )
 
+        if self.params.geometry.convert_stills_to_sequences:
+            if any(not isinstance(i, ImageSequence) for i in experiments.imagesets()):
+                files_to_indiv = defaultdict(int)
+                beams = []
+                formats = []
+                detectors = []
+                iset_params = []
+                existing_isets_sequences = []
+                for i, iset in enumerate(experiments.imagesets()):
+                    if not isinstance(iset, ImageSequence):
+                        path = iset.get_path(0)
+                        if path not in files_to_indiv:
+                            beams.append(iset.get_beam())
+                            detectors.append(iset.get_detector())
+                            formats.append(iset.get_format_class())
+                            iset_params.append(iset.params())
+                        files_to_indiv[iset.get_path(0)] += 1
+                    else:
+                        existing_isets_sequences.append(iset)
+
+                from dxtbx.imageset import ImageSetFactory
+                from dxtbx.model import GoniometerFactory, Scan
+
+                new_experiments = ExperimentList()
+                for i, (file, n) in enumerate(files_to_indiv.items()):
+                    if self.params.geometry.scan.image_range:
+                        user_start, user_end = self.params.geometry.scan.image_range
+                        first, last = user_start, user_end
+                    else:
+                        first, last = 1, n
+                    iset_params[i].update({"lazy": False})
+                    sequence = ImageSetFactory.make_sequence(
+                        template=file,
+                        indices=list(range(first, last + 1)),
+                        format_class=formats[i],
+                        beam=beams[i],
+                        detector=detectors[i],
+                        goniometer=GoniometerFactory.make_goniometer(
+                            (0.0, 1.0, 0.0), (1, 0, 0, 0, 1, 0, 0, 0, 1)
+                        ),
+                        scan=Scan(image_range=(first, last), oscillation=(0.0, 0.0)),
+                        format_kwargs=iset_params[i],
+                    )
+                    sequence = self.update_lookup(sequence, lookup)  # for mask etc
+                    for j in range(len(sequence)):
+                        subset = sequence[j : j + 1]
+                        new_experiments.append(
+                            Experiment(
+                                imageset=sequence,
+                                beam=sequence.get_beam(),
+                                detector=sequence.get_detector(),
+                                goniometer=sequence.get_goniometer(),
+                                scan=subset.get_scan(),
+                                crystal=None,
+                            )
+                        )
+                if existing_isets_sequences:
+                    for expt in experiments:
+                        if expt.imageset in existing_isets_sequences:
+                            new_experiments.append(expt)
+                experiments = new_experiments
+
         if self.params.identifier_type:
             generate_experiment_identifiers(experiments, self.params.identifier_type)
         # Return the experiments
@@ -891,12 +935,27 @@ def do_import(
         num_images += len(e.imageset)
         counted_imagesets.append(e.imageset)
 
-    format_list = {str(e.imageset.get_format_class()) for e in experiments}
+    unique_formats = OrderedSet()
+    unique_templates = OrderedSet()
+    for imgset in counted_imagesets:
+        unique_formats.add(imgset.get_format_class())
+        if scan := imgset.get_scan():
+            start, end = scan.get_image_range()
+            unique_templates.add(f"{imgset.get_template()}:{start}:{end}")
+        else:
+            paths = imgset.reader().paths()
+            if len(paths) == 1:
+                unique_templates.add(paths[0])
+            else:
+                template, _ = template_regex_from_list(paths)
+                unique_templates.add(template)
 
     # Print out some bulk info
     logger.info("-" * 80)
-    for f in format_list:
-        logger.info("  format: %s", f)
+    for fmt in unique_formats:
+        logger.info("  format: %s", fmt)
+    for template in unique_templates:
+        logger.info("  template: %s", template)
     logger.info("  num images: %d", num_images)
     logger.info("  sequences:")
     logger.info("    still:    %d", num_still_sequences)
