@@ -25,6 +25,7 @@ from __future__ import annotations
 import logging
 import math
 import sys
+import warnings
 
 from orderedset import OrderedSet
 
@@ -37,6 +38,7 @@ from dials.algorithms.profile_model.factory import ProfileModelFactory
 from dials.array_family import flex
 from dials.util import show_mail_handle_errors
 from dials.util.command_line import heading
+from dials.util.exclude_images import expand_exclude_multiples, set_invalid_images
 from dials.util.options import ArgumentParser, reflections_and_experiments_from_files
 from dials.util.slice import slice_crystal
 from dials.util.version import dials_version
@@ -123,10 +125,7 @@ phil_scope = parse(
 
   }
 
-  exclude_images = None
-    .type = ints
-    .help = "Exclude images from integration (e.g. 1,2,3,4,5 etc)"
-
+  include scope dials.util.exclude_images.phil_scope
   include scope dials.algorithms.integration.integrator.phil_scope
   include scope dials.algorithms.profile_model.factory.phil_scope
   include scope dials.algorithms.spot_prediction.reflection_predictor.phil_scope
@@ -476,11 +475,38 @@ def run_integration(params, experiments, reference=None):
             experiments, reference, params.scan_range
         )
 
-    # Modify experiment list if exclude images is set
+    # Modify experiment list if exclude_images is set
+    if params.exclude_images_multiple:
+        params.exclude_images = expand_exclude_multiples(
+            experiments,
+            params.exclude_images_multiple,
+            params.exclude_images,
+        )
     if params.exclude_images:
-        for experiment in experiments:
-            for index in params.exclude_images:
-                experiment.imageset.mark_for_rejection(index, True)
+        try:
+            experiments = set_invalid_images(experiments, params.exclude_images)
+        except ValueError as err:
+            # Handle deprecated way of providing exclude_images
+            try:
+                exclude_images = [
+                    int(e)
+                    for e in str(params.exclude_images)
+                    .replace("[", "")
+                    .replace("]", "")
+                    .replace("'", "")
+                    .split(",")
+                ]
+            except ValueError:
+                raise (err)
+            warnings.warn(
+                "Providing exclude_images as a single list (e.g. 1,2,3,4,5 etc.) is deprecated.\n"
+                + str(err),
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            for experiment in experiments:
+                for index in exclude_images:
+                    experiment.imageset.mark_for_rejection(index, True)
 
     # Predict the reflections
     logger.info("\n".join(("", "=" * 80, "")))
@@ -607,11 +633,22 @@ def run_integration(params, experiments, reference=None):
 
     # Correct integrated intensities for absorption correction, if necessary
     for abs_params in params.absorption_correction:
-        if abs_params.apply and abs_params.algorithm == "fuller_kapton":
-            from dials.algorithms.integration.kapton_correction import (
-                multi_kapton_correction,
-            )
-
+        if abs_params.apply:
+            if abs_params.algorithm == "fuller_kapton":
+                from dials.algorithms.integration.kapton_correction import (
+                    multi_kapton_correction,
+                )
+            elif abs_params.algorithm == "kapton_2019":
+                from dials.algorithms.integration.kapton_2019_correction import (
+                    multi_kapton_correction,
+                )
+            elif abs_params.algorithm == "other":
+                continue  # custom abs. corr. implementation should go here
+            else:
+                raise ValueError(
+                    "absorption_correction.apply=True, "
+                    "but no .algorithm has been selected!"
+                )
             experiments, reflections = multi_kapton_correction(
                 experiments, reflections, abs_params.fuller_kapton, logger=logger
             )()
