@@ -124,6 +124,10 @@ def _control_phil_str():
     logging_dir = None
       .type = str
       .help = Directory output log files will be placed
+    logging_option = normal *suppressed disabled
+      .type = choice
+      .help = normal includes all logging, suppress turns off DIALS refine output
+      .help = and disabled removes basically all logging
     experiments_filename = None
       .type = str
       .help = The filename for output experiments. For example, %s_imported.expt
@@ -152,9 +156,10 @@ def _control_phil_str():
     profile_filename = None
       .type = str
       .help = The filename for output reflection profile parameters
-    integration_pickle = int-%d-%s.pickle
+    integration_pickle = None
       .type = str
-      .help = Filename for cctbx.xfel-style integration pickle files
+      .expert_level = 3
+      .help = Filename for legacy cxi.merge integration pickle files. Example: int-%d-%s.pickle
   }
 
   mp {
@@ -226,6 +231,29 @@ def _dials_phil_str():
         .type = strings
         .help = List of indexing methods. If indexing fails with first method, indexing will be \
                 attempted with the next, and so forth
+      reflection_subsampling
+      {
+        enable  = False
+          .type = bool
+          .help = Enable random subsampling of reflections during indexing. Attempts to index    \
+                  will be repeated with random subsampling of spotfinder spots, starting at      \
+                  step_start % of total spots, repeating n_attempts_per_step times per step,     \
+                  and decreasing by step_size % until step_stop % is reached. With the defaults, \
+                  26 total indexing attempts will be made, randomly subsampling from 100% to     \
+                  50% by 2 % steps, one attempt per step.
+        step_start = 100
+          .type = int
+          .help = What percent of reflections to start with
+        step_stop = 50
+          .type = int
+          .help = What percent of reflections to end with
+        step_size = 2
+          .type = int
+          .help = What percent of reflections to decrease by per step
+        n_attempts_per_step = 1
+          .type = int
+          .help = How many attempts to make at each step
+      }
       known_orientations = None
         .type = path
         .multiple = True
@@ -468,6 +496,23 @@ class Script:
                 transmitted_info = None
             params, options, all_paths = comm.bcast(transmitted_info, root=0)
 
+        if params.output.logging_option == "suppressed":
+            logging.getLogger("dials.algorithms.indexing.nave_parameters").setLevel(
+                logging.ERROR
+            )
+            logging.getLogger("dials.algorithms.indexing.stills_indexer").setLevel(
+                logging.ERROR
+            )
+            logging.getLogger("dials.algorithms.refinement.refiner").setLevel(
+                logging.ERROR
+            )
+            logging.getLogger(
+                "dials.algorithms.refinement.reflection_manager"
+            ).setLevel(logging.ERROR)
+
+        elif params.output.logging_option == "disabled":
+            logging.disable(logging.ERROR)
+
         # Check we have some filenames
         if not all_paths:
             self.parser.print_help()
@@ -479,7 +524,7 @@ class Script:
             self.pr = cProfile.Profile()
             self.pr.enable()
 
-        print(f"Have {len(all_paths)} files")
+        logger.info(f"Have {len(all_paths)} files")
 
         # Mask validation
         for mask_path in params.spotfinder.lookup.mask, params.integration.lookup.mask:
@@ -745,7 +790,12 @@ class Script:
 
                 if rank == 0:
                     # server process
+                    num_iter = len(iterable)
                     for item_num, item in enumerate(iterable):
+                        print(
+                            "Processing %d / %d shots" % (item_num, num_iter),
+                            flush=True,
+                        )
                         if process_fractions and not process_this_event(item_num):
                             continue
 
@@ -954,9 +1004,9 @@ class Processor:
         if not self.params.mp.debug.output_debug_logs:
             return
 
-        from xfel.cxi.cspad_ana import cspad_tbx  # XXX move to common timestamp format
+        from serialtbx.util.time import timestamp  # XXX move to common timestamp format
 
-        ts = cspad_tbx.evt_timestamp()  # Now
+        ts = timestamp()  # Now
         debug_file_handle = open(self.debug_file_path, "a")
         if string == "":
             debug_file_handle.write("\n")
@@ -1190,33 +1240,58 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                 if self.params.indexing.stills.require_known_orientation:
                     raise
 
-        if params.indexing.stills.method_list is None and not indexing_succeeded:
-            idxr = Indexer.from_parameters(
-                reflections,
-                experiments,
-                params=params,
-            )
-            idxr.index()
-        elif not indexing_succeeded:
-            indexing_error = None
-            for method in params.indexing.stills.method_list:
-                params.indexing.method = method
+        if not indexing_succeeded:
+            if self.params.indexing.stills.reflection_subsampling.enable:
+                subsets = range(
+                    self.params.indexing.stills.reflection_subsampling.step_start,
+                    self.params.indexing.stills.reflection_subsampling.step_stop
+                    - self.params.indexing.stills.reflection_subsampling.step_size,
+                    -self.params.indexing.stills.reflection_subsampling.step_size,
+                )
+            else:
+                subsets = [100]
+            all_reflections = reflections
+            subset_indexing_error = None
+            for i in subsets:
+                if i != 100:
+                    reflections = all_reflections.select(
+                        flex.random_permutation(len(all_reflections))
+                    )[: int(len(all_reflections) * i / 100)]
                 try:
-                    idxr = Indexer.from_parameters(
-                        reflections, experiments, params=params
-                    )
-                    idxr.index()
-                except Exception as e:
-                    logger.info("Couldn't index using method %s", method)
-                    if indexing_error is None:
-                        if e is None:
-                            e = Exception(f"Couldn't index using method {method}")
-                        indexing_error = e
+                    if params.indexing.stills.method_list is None:
+                        idxr = Indexer.from_parameters(
+                            reflections,
+                            experiments,
+                            params=params,
+                        )
+                        idxr.index()
+                    else:
+                        ml_indexing_error = None
+                        for method in params.indexing.stills.method_list:
+                            params.indexing.method = method
+                            try:
+                                idxr = Indexer.from_parameters(
+                                    reflections,
+                                    experiments,
+                                    params=params,
+                                )
+                                idxr.index()
+                            except Exception as e_ml:
+                                logger.info("Couldn't index using method %s", method)
+                                ml_indexing_error = e_ml
+                            else:
+                                ml_indexing_error = None
+                                break
+                        if ml_indexing_error:
+                            raise ml_indexing_error
+                except Exception as e_subset:
+                    subset_indexing_error = e_subset
                 else:
-                    indexing_error = None
+                    logger.info("Indexed using %d%% of the reflections" % i)
+                    subset_indexing_error = None
                     break
-            if indexing_error is not None:
-                raise indexing_error
+            if subset_indexing_error:
+                raise subset_indexing_error
 
         indexed = idxr.refined_reflections
         experiments = idxr.refined_experiments
@@ -1332,7 +1407,7 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                     )
 
         if self.params.dispatch.coset:
-            from xfel.util.sublattice_helper import integrate_coset
+            from dials.algorithms.integration.sublattice_helper import integrate_coset
 
             integrate_coset(self, experiments, indexed)
 
@@ -1400,7 +1475,13 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                     from dials.algorithms.integration.kapton_2019_correction import (
                         multi_kapton_correction,
                     )
-
+                elif abs_params.algorithm == "other":
+                    continue  # custom abs. corr. implementation should go here
+                else:
+                    raise ValueError(
+                        "absorption_correction.apply=True, "
+                        "but no .algorithm has been selected!"
+                    )
                 experiments, integrated = multi_kapton_correction(
                     experiments, integrated, abs_params.fuller_kapton, logger=logger
                 )()
@@ -1523,7 +1604,7 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
             return
 
         if self.params.output.integration_pickle is not None:
-            from xfel.command_line.frame_extractor import ConstructFrame
+            from serialtbx.util.construct_frame import ConstructFrame
 
             # Split everything into separate experiments for pickling
             for e_number, experiment in enumerate(experiments):
