@@ -285,7 +285,7 @@ def get_subset_for_symmetry(experiments, reflection_tables, exclude_images=None)
     else:
         for expt, refl in zip(experiments, reflection_tables):
             sel = get_selection_for_valid_image_ranges(refl, expt)
-            if not sel.count(False):
+            if expt.scan and not sel.count(False):
                 # Use first 360 degrees if <360 deg i.e. first measured data,
                 # but only if no reflections have been explicitly excluded
                 # already
