@@ -5,8 +5,10 @@ Test the command line script dials.scale, for successful completion.
 from __future__ import annotations
 
 import json
+import os
+import shutil
+import subprocess
 
-import procrunner
 import pytest
 
 import iotbx.merging_statistics
@@ -25,9 +27,9 @@ from dials.util.options import ArgumentParser
 
 def run_one_scaling(working_directory, argument_list):
     """Run the dials.scale algorithm."""
-    command = ["dials.scale"] + argument_list
+    command = [shutil.which("dials.scale")] + argument_list
     print(command)
-    result = procrunner.run(command, working_directory=working_directory)
+    result = subprocess.run(command, cwd=working_directory, capture_output=True)
     print(result.stderr)
     assert not result.returncode and not result.stderr
     assert (working_directory / "scaled.expt").is_file()
@@ -255,9 +257,9 @@ def test_targeted_scaling_against_mtz(dials_data, tmp_path):
     location = dials_data("l_cysteine_4_sweeps_scaled", pathlib=True)
     refl = location / "scaled_35.refl"
     expt = location / "scaled_35.expt"
-    command = ["dials.scale", refl, expt, "unmerged_mtz=unmerged.mtz"]
+    command = [shutil.which("dials.scale"), refl, expt, "unmerged_mtz=unmerged.mtz"]
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.expt").is_file()
     assert (tmp_path / "scaled.refl").is_file()
@@ -267,14 +269,14 @@ def test_targeted_scaling_against_mtz(dials_data, tmp_path):
     expt = location / "scaled_30.expt"
     reference = tmp_path / "unmerged.mtz"
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         refl,
         expt,
         f"reference={reference}",
         "unmerged_mtz=unmerged_2.mtz",
     ]
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.expt").is_file()
     assert (tmp_path / "scaled.refl").is_file()
@@ -315,13 +317,13 @@ def vmxi_protk_reindexed(dials_data, tmp_path_factory):
     location = dials_data("vmxi_proteinase_k_sweeps", pathlib=True)
 
     command = [
-        "dials.reindex",
+        shutil.which("dials.reindex"),
         location / "experiments_0.json",
         location / "reflections_0.pickle",
         "space_group=P422",
     ]
     tmp_path = tmp_path_factory.mktemp("vmxi_protk_reindexed")
-    procrunner.run(command, working_directory=tmp_path)
+    subprocess.run(command, cwd=tmp_path, capture_output=True)
     return tmp_path / "reindexed.expt", tmp_path / "reindexed.refl"
 
 
@@ -420,6 +422,7 @@ def test_scale_physical(dials_data, tmp_path):
         "use_free_set=1",
         "outlier_rejection=simple",
         "json=scaling.json",
+        "additional_stats=True",
     ]
     run_one_scaling(tmp_path, [refl_1, expt_1] + extra_args)
     unmerged_mtz = tmp_path / "unmerged.mtz"
@@ -447,6 +450,36 @@ def test_scale_physical(dials_data, tmp_path):
         False
     )
 
+    # run with a precalculated analytical correction (set to 1 for test to check result the same)
+    data_dir = dials_data("l_cysteine_dials_output", pathlib=True)
+    r2 = flex.reflection_table.from_file(data_dir / "20_integrated.pickle")
+    r2["analytical_correction"] = flex.double(r2.size(), 1.0)
+    r2.as_file(tmp_path / "modified.refl")
+    extra_args = [
+        "model=physical",
+        "physical.analytical_correction=True",
+        "error_model=None",
+        "intensity_choice=profile",
+        "unmerged_mtz=unmerged.mtz",
+        "use_free_set=1",
+        "outlier_rejection=simple",
+    ]
+    import os
+
+    run_one_scaling(
+        tmp_path, [os.fspath(tmp_path / "modified.refl"), expt_1] + extra_args
+    )
+    unmerged_mtz = tmp_path / "unmerged.mtz"
+    assert unmerged_mtz.is_file()
+    expts = load.experiment_list(tmp_path / "scaled.expt", check_format=False)
+    assert "analytical" in expts[0].scaling_model.components
+
+    # Now inspect output, check it's identical
+    result2 = get_merging_stats(unmerged_mtz)
+    assert result2.overall.r_pim == result.overall.r_pim
+    assert result2.overall.cc_one_half == result.overall.cc_one_half
+    assert result2.overall.n_obs == result.overall.n_obs
+
     # run again with the concurrent scaling option turned off and the 'standard'
     # outlier rejection
     extra_args = [
@@ -482,13 +515,13 @@ def test_scale_set_absorption_level(dials_data, tmp_path):
     # minimiser due to indeterminate solution of the normal equations. In this
     # case, the error should be caught and scaling can proceed.
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         refl,
         expt,
         "absorption_level=medium",
         "unmerged_mtz=unmerged.mtz",
     ]
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -504,13 +537,13 @@ def test_scale_set_absorption_level(dials_data, tmp_path):
     ## now scale again with different options, but fix the absorption surface to
     # test the correction.fix option.
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         tmp_path / "scaled.refl",
         tmp_path / "scaled.expt",
         "error_model=None",
         "physical.correction.fix=absorption",
     ]
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -527,8 +560,8 @@ def test_scale_normal_equations_failure(dials_data, tmp_path):
     # exclude a central region of data to force the failure of the full matrix
     # minimiser due to indeterminate solution of the normal equations. In this
     # case, the error should be caught and scaling can proceed.
-    command = ["dials.scale", refl, expt, "exclude_images=800:1400"]
-    result = procrunner.run(command, working_directory=tmp_path)
+    command = [shutil.which("dials.scale"), refl, expt, "exclude_images=800:1400"]
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -539,7 +572,7 @@ def test_scale_and_filter_image_group_mode(dials_data, tmp_path):
     location = dials_data("multi_crystal_proteinase_k", pathlib=True)
 
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         "filtering.method=deltacchalf",
         "stdcutoff=4.0",
         "mode=image_group",
@@ -555,7 +588,7 @@ def test_scale_and_filter_image_group_mode(dials_data, tmp_path):
         command.append(location / f"experiments_{i}.json")
         command.append(location / f"reflections_{i}.pickle")
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -580,7 +613,7 @@ def test_scale_and_filter_termination(dials_data, tmp_path):
     location = dials_data("multi_crystal_proteinase_k", pathlib=True)
 
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         "filtering.method=deltacchalf",
         "stdcutoff=2.0",
         "max_percent_removed=5.0",
@@ -595,7 +628,7 @@ def test_scale_and_filter_termination(dials_data, tmp_path):
         command.append(location / f"experiments_{i}.json")
         command.append(location / f"reflections_{i}.pickle")
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -615,7 +648,7 @@ def test_scale_and_filter_image_group_single_dataset(dials_data, tmp_path):
     single data set."""
     data_dir = dials_data("l_cysteine_dials_output", pathlib=True)
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         data_dir / "20_integrated.pickle",
         data_dir / "20_integrated_experiments.json",
         "filtering.method=deltacchalf",
@@ -625,7 +658,7 @@ def test_scale_and_filter_image_group_single_dataset(dials_data, tmp_path):
         "scale_and_filter_results=analysis_results.json",
         "error_model=None",
     ]
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -646,12 +679,12 @@ def test_scale_when_a_dataset_is_filtered_out(dials_data, tmp_path):
     refls = flex.reflection_table.from_file(location / "reflections_3.pickle")
     refls["partiality"] = flex.double(refls.size(), 0.3)
     refls.as_file(tmp_path / "modified_3.refl")
-    command = ["dials.scale", "d_min=2.0", tmp_path / "modified_3.refl"]
+    command = [shutil.which("dials.scale"), "d_min=2.0", tmp_path / "modified_3.refl"]
     for i in [1, 2, 3, 4]:
         command.append(location / f"experiments_{i}.json")
     for i in [1, 2, 4]:
         command.append(location / f"reflections_{i}.pickle")
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     expts = load.experiment_list(tmp_path / "scaled.expt", check_format=False)
@@ -661,12 +694,12 @@ def test_scale_when_a_dataset_is_filtered_out(dials_data, tmp_path):
 def test_scale_dose_decay_model(dials_data, tmp_path):
     """Test the scale and filter command line program."""
     location = dials_data("multi_crystal_proteinase_k", pathlib=True)
-    command = ["dials.scale", "d_min=2.0", "model=dose_decay"]
+    command = [shutil.which("dials.scale"), "d_min=2.0", "model=dose_decay"]
     for i in [1, 2, 3, 4, 5, 7, 10]:
         command.append(location / f"experiments_{i}.json")
         command.append(location / f"reflections_{i}.pickle")
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -680,7 +713,7 @@ def test_scale_best_unit_cell_d_min(dials_data, tmp_path):
     best_unit_cell = uctbx.unit_cell((42, 42, 39, 90, 90, 90))
     d_min = 2
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         "best_unit_cell=%g,%g,%g,%g,%g,%g" % best_unit_cell.parameters(),
         f"d_min={d_min:g}",
         "unmerged_mtz=unmerged.mtz",
@@ -689,7 +722,7 @@ def test_scale_best_unit_cell_d_min(dials_data, tmp_path):
     for i in [1, 2, 3, 4, 5, 7, 10]:
         command.append(location / f"experiments_{i}.json")
         command.append(location / f"reflections_{i}.pickle")
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "scaled.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -709,7 +742,7 @@ def test_scale_and_filter_dataset_mode(dials_data, tmp_path):
     """Test the scale and filter command line program."""
     location = dials_data("multi_crystal_proteinase_k", pathlib=True)
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         "filtering.method=deltacchalf",
         "stdcutoff=1.0",
         "mode=dataset",
@@ -724,7 +757,7 @@ def test_scale_and_filter_dataset_mode(dials_data, tmp_path):
         command.append(location / f"experiments_{i}.json")
         command.append(location / f"reflections_{i}.pickle")
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "filtered.refl").is_file()
     assert (tmp_path / "scaled.expt").is_file()
@@ -923,7 +956,7 @@ def test_scale_handle_bad_dataset(dials_data, tmp_path):
     scaling job completes without failure."""
     location = dials_data("multi_crystal_proteinase_k", pathlib=True)
     command = [
-        "dials.scale",
+        shutil.which("dials.scale"),
         "reflection_selection.method=intensity_ranges",
         "Isigma_range=90.0,1000",
     ]
@@ -931,7 +964,7 @@ def test_scale_handle_bad_dataset(dials_data, tmp_path):
         command.append(location / f"experiments_{i}.json")
         command.append(location / f"reflections_{i}.pickle")
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
 
     reflections = flex.reflection_table.from_file(tmp_path / "scaled.refl")
@@ -940,6 +973,31 @@ def test_scale_handle_bad_dataset(dials_data, tmp_path):
     assert len(reflections.experiment_identifiers()) == 4
 
 
+def test_target_scale_handle_bad_dataset(dials_data, tmp_path):
+    """Set command line parameters such that some datasets do not meet the
+    criteria for inclusion in scaling. Check that these are excluded and the
+    scaling job completes without failure."""
+    location = dials_data("cunir_serial_processed", pathlib=True)
+    pdb = dials_data("cunir_serial", pathlib=True) / "2BW4.pdb"
+    command = [
+        shutil.which("dials.scale"),
+        "reflection_selection.method=intensity_ranges",
+        "Isigma_range=20.0,0.0",
+        "full_matrix=None",
+        os.fspath(location / "integrated.refl"),
+        os.fspath(location / "integrated.expt"),
+        f"reference={os.fspath(pdb)}",
+    ]
+
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
+    assert not result.returncode and not result.stderr
+
+    reflections = flex.reflection_table.from_file(tmp_path / "scaled.refl")
+    expts = load.experiment_list(tmp_path / "scaled.expt", check_format=False)
+    assert len(expts) == 2
+    assert len(reflections.experiment_identifiers()) == 2
+
+
 def test_targeted_scaling(dials_data, tmp_path):
     """Test the targeted scaling workflow."""
     location = dials_data("l_cysteine_4_sweeps_scaled", pathlib=True)
@@ -1005,9 +1063,15 @@ def test_incremental_scale_workflow(dials_data, tmp_path):
     # test order also - first new file before scaled
     refl_2 = data_dir / "25_integrated.pickle"
     expt_2 = data_dir / "25_integrated_experiments.json"
-    command = ["dials.cosym", refl_2, expt_2, "scaled.refl", "scaled.expt"]
+    command = [
+        shutil.which("dials.cosym"),
+        refl_2,
+        expt_2,
+        "scaled.refl",
+        "scaled.expt",
+    ]
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "symmetrized.expt").is_file()
     assert (tmp_path / "symmetrized.refl").is_file()
@@ -1019,7 +1083,7 @@ def test_incremental_scale_workflow(dials_data, tmp_path):
     refl_2 = data_dir / "30_integrated.pickle"
     expt_2 = data_dir / "30_integrated_experiments.json"
     command = [
-        "dials.cosym",
+        shutil.which("dials.cosym"),
         "scaled.refl",
         "scaled.expt",
         refl_1,
@@ -1027,7 +1091,7 @@ def test_incremental_scale_workflow(dials_data, tmp_path):
         "output.reflections=symmetrized.refl",
         "output.experiments=symmetrized.expt",
     ]
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert (tmp_path / "symmetrized.expt").is_file()
     assert (tmp_path / "symmetrized.refl").is_file()
@@ -1059,8 +1123,8 @@ def test_scale_cross_validate(dials_data, tmp_path, mode, parameter, parameter_v
         extra_args += [f"parameter={parameter}"]
     if parameter_values:
         extra_args += [f"parameter_values={parameter_values}"]
-    command = ["dials.scale", refl, expt] + extra_args
-    result = procrunner.run(command, working_directory=tmp_path)
+    command = [shutil.which("dials.scale"), refl, expt] + extra_args
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
 
 
