@@ -5,6 +5,7 @@ import logging
 import os
 import pickle
 import random
+from pathlib import Path
 
 import pytest
 
@@ -12,7 +13,9 @@ from cctbx import sgtbx
 from dxtbx.model import Crystal, Experiment, ExperimentList
 from dxtbx.serialize import load
 
+from dials.algorithms.shoebox import MaskCode
 from dials.array_family import flex
+from dials.model.data import Shoebox
 
 
 def test_accessing_invalid_key_throws_keyerror():
@@ -172,6 +175,17 @@ def test_delete():
     assert table.ncols() == 2
 
 
+def test_slice():
+    table = flex.reflection_table()
+    table["col1"] = flex.int([0, 0, 1, 2, 3, 4, 5])
+    table["id"] = flex.int([-1, 0, 1, 2, 3, 4, 5])
+    for i in range(0, 6):
+        table.experiment_identifiers()[i] = str(i)
+    sliced = table[4:]
+    assert sliced.nrows() == 3
+    assert dict(sliced.experiment_identifiers()) == {3: "3", 4: "4", 5: "5"}
+
+
 def test_row_operations():
     # The columns as lists
     c1 = list(range(10))
@@ -745,8 +759,6 @@ def test_copy():
 
 
 def test_extract_shoeboxes():
-    from dials.algorithms.shoebox import MaskCode
-
     random.seed(0)
 
     reflections = flex.reflection_table()
@@ -940,8 +952,6 @@ def test_split_partials():
 
 
 def test_split_partials_with_shoebox():
-    from dials.model.data import Shoebox
-
     r = flex.reflection_table()
     r["value1"] = flex.double()
     r["value2"] = flex.int()
@@ -1085,9 +1095,7 @@ def test_find_overlapping():
             assert is_overlap(b0, b1, i)
 
 
-def test_to_from_msgpack(tmpdir):
-    from dials.model.data import Shoebox
-
+def test_to_from_msgpack(tmp_path):
     def gen_shoebox():
         shoebox = Shoebox(0, (0, 4, 0, 3, 0, 1))
         shoebox.allocate()
@@ -1159,10 +1167,8 @@ def test_to_from_msgpack(tmpdir):
     assert all(tuple(a == b for a, b in zip(new_table["col10"], c10)))
     assert all(tuple(compare(a, b) for a, b in zip(new_table["col11"], c11)))
 
-    table.as_msgpack_file(tmpdir.join("reflections.mpack").strpath)
-    new_table = flex.reflection_table.from_msgpack_file(
-        tmpdir.join("reflections.mpack").strpath
-    )
+    table.as_msgpack_file(tmp_path / "reflections.mpack")
+    new_table = flex.reflection_table.from_msgpack_file(tmp_path / "reflections.mpack")
     assert new_table.is_consistent()
     assert new_table.nrows() == 10
     assert new_table.ncols() == 11
@@ -1180,8 +1186,6 @@ def test_to_from_msgpack(tmpdir):
 
 
 def test_experiment_identifiers():
-    from dxtbx.model import Experiment, ExperimentList
-
     table = flex.reflection_table()
     table["id"] = flex.int([0, 1, 2, 3])
 
@@ -1295,8 +1299,8 @@ def test_experiment_identifiers():
 
 
 def test_select_remove_on_experiment_identifiers(caplog):
-
     table = flex.reflection_table()
+    table.reset_ids()  # test empty table
     table["id"] = flex.int([0, 1, 2, 3])
 
     experiments = ExperimentList()
@@ -1382,7 +1386,7 @@ def test_as_miller_array():
         _ = table.as_miller_array(experiment, intensity="2")
 
 
-def test_map_centroids_to_reciprocal_space(dials_regression):
+def test_map_centroids_to_reciprocal_space(dials_regression: Path):
     data_dir = os.path.join(dials_regression, "indexing_test_data", "i04_weak_data")
     pickle_path = os.path.join(data_dir, "full.pickle")
     expts_path = os.path.join(data_dir, "experiments_import.json")
@@ -1439,7 +1443,7 @@ def test_map_centroids_to_reciprocal_space(dials_regression):
     )
 
 
-def test_calculate_entering_flags(dials_regression):
+def test_calculate_entering_flags(dials_regression: Path):
     data_dir = os.path.join(dials_regression, "indexing_test_data", "i04_weak_data")
     pickle_path = os.path.join(data_dir, "full.pickle")
     experiments_path = os.path.join(data_dir, "experiments_import.json")
@@ -1584,7 +1588,6 @@ def test_match_by_hkle():
 
 
 def test_concat():
-
     table1 = flex.reflection_table()
     table2 = flex.reflection_table()
 
@@ -1604,3 +1607,8 @@ def test_concat():
     assert list(table1["id"]) == [0, 0, 1, 1, 2, 2, 3, 3]
     assert list(ids1.keys()) == [0, 1, 2, 3]
     assert list(ids1.values()) == ["a", "b", "c", "d"]
+
+    # test empty tables
+    table1 = flex.reflection_table()
+    table2 = flex.reflection_table()
+    table1 = flex.reflection_table.concat([table1, table2])
