@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-import procrunner
+import shutil
+import subprocess
 
 
 def test_standalone_error_model_refinement_on_scaled_data(dials_data, tmp_path):
@@ -10,7 +11,7 @@ def test_standalone_error_model_refinement_on_scaled_data(dials_data, tmp_path):
 
     # Use a range of options, some default.
     command = [
-        "dials.refine_error_model",
+        shutil.which("dials.refine_error_model"),
         refls,
         expts,
         "json=error_model.json",
@@ -20,7 +21,7 @@ def test_standalone_error_model_refinement_on_scaled_data(dials_data, tmp_path):
         "basic.minimisation=individual",
     ]
 
-    result = procrunner.run(command, working_directory=tmp_path)
+    result = subprocess.run(command, cwd=tmp_path, capture_output=True)
     assert not result.returncode and not result.stderr
     assert tmp_path.joinpath("error_model.html").is_file()
     assert tmp_path.joinpath("error_model.json").is_file()
